@@ -9,8 +9,4 @@ export const MIN_PANEL_HEIGHT = GRID_CELL_HEIGHT * 3;
 
 export const LS_PANEL_COPY_KEY = 'panel-copy';
 
-export const DASHBOARD_TOOLBAR_HEIGHT = 55;
-export const DASHBOARD_TOP_PADDING = 20;
-
-export const PANEL_HEADER_HEIGHT = 27;
 export const PANEL_BORDER = 2;
