@@ -1,22 +1,30 @@
 import _ from 'lodash';
+import { NavModelSrv } from 'app/core/core';
+import { ILocationService } from 'angular';
+import { BackendSrv } from '@grafana/runtime';
 
 export class SnapshotListCtrl {
   navModel: any;
   snapshots: any;
 
   /** @ngInject */
-  constructor(private $rootScope, private backendSrv, navModelSrv, private $location) {
+  constructor(
+    private $rootScope: any,
+    private backendSrv: BackendSrv,
+    navModelSrv: NavModelSrv,
+    private $location: ILocationService
+  ) {
     this.navModel = navModelSrv.getNav('dashboards', 'snapshots', 0);
-    this.backendSrv.get('/api/dashboard/snapshots').then(result => {
+    this.backendSrv.get('/api/dashboard/snapshots').then((result: any) => {
       const baseUrl = this.$location.absUrl().replace($location.url(), '');
-      this.snapshots = result.map(snapshot => ({
+      this.snapshots = result.map((snapshot: any) => ({
         ...snapshot,
         url: snapshot.externalUrl || `${baseUrl}/dashboard/snapshot/${snapshot.key}`,
       }));
     });
   }
 
-  removeSnapshotConfirmed(snapshot) {
+  removeSnapshotConfirmed(snapshot: any) {
     _.remove(this.snapshots, { key: snapshot.key });
     this.backendSrv.delete('/api/snapshots/' + snapshot.key).then(
       () => {},
@@ -26,7 +34,7 @@ export class SnapshotListCtrl {
     );
   }
 
-  removeSnapshot(snapshot) {
+  removeSnapshot(snapshot: any) {
     this.$rootScope.appEvent('confirm-modal', {
       title: 'Delete',
       text: 'Are you sure you want to delete snapshot ' + snapshot.name + '?',
