@@ -8,7 +8,9 @@ import { loadPlugins, setPluginsLayoutMode, setPluginsSearchQuery } from './stat
 import { getNavModel } from 'app/core/selectors/navModel';
 import { getLayoutMode, getPlugins, getPluginsSearchQuery } from './state/selectors';
 import { LayoutMode } from 'app/core/components/LayoutSelector/LayoutSelector';
-import { PluginMeta, NavModel } from '@grafana/ui';
+import { NavModel } from '@grafana/data';
+import { PluginMeta } from '@grafana/ui';
+import { StoreState } from 'app/types';
 
 export interface Props {
   navModel: NavModel;
@@ -65,7 +67,7 @@ export class PluginListPage extends PureComponent<Props> {
   }
 }
 
-function mapStateToProps(state) {
+function mapStateToProps(state: StoreState) {
   return {
     navModel: getNavModel(state.navIndex, 'plugins'),
     plugins: getPlugins(state.plugins),
