@@ -1,21 +1,25 @@
 import kbn from 'app/core/utils/kbn';
 import _ from 'lodash';
 import { variableRegex } from 'app/features/templating/variable';
-import { ScopedVars } from '@grafana/ui';
-import { TimeRange } from '@grafana/data';
+import { TimeRange, ScopedVars } from '@grafana/data';
 
-function luceneEscape(value) {
+function luceneEscape(value: string) {
   return value.replace(/([\!\*\+\-\=<>\s\&\|\(\)\[\]\{\}\^\~\?\:\\/"])/g, '\\$1');
 }
 
+interface FieldAccessorCache {
+  [key: string]: (obj: any) => any;
+}
+
 export class TemplateSrv {
   variables: any[];
 
   private regex = variableRegex;
-  private index = {};
-  private grafanaVariables = {};
+  private index: any = {};
+  private grafanaVariables: any = {};
   private builtIns: any = {};
   private timeRange: TimeRange = null;
+  private fieldAccessorCache: FieldAccessorCache = {};
 
   constructor() {
     this.builtIns['__interval'] = { text: '1s', value: '1s' };
@@ -23,7 +27,7 @@ export class TemplateSrv {
     this.variables = [];
   }
 
-  init(variables, timeRange?: TimeRange) {
+  init(variables: any, timeRange?: TimeRange) {
     this.variables = variables;
     this.timeRange = timeRange;
     this.updateIndex();
@@ -34,7 +38,7 @@ export class TemplateSrv {
   }
 
   updateIndex() {
-    const existsOrEmpty = value => value || value === '';
+    const existsOrEmpty = (value: any) => value || value === '';
 
     this.index = this.variables.reduce((acc, currentValue) => {
       if (currentValue.current && (currentValue.current.isNone || existsOrEmpty(currentValue.current.value))) {
@@ -64,12 +68,12 @@ export class TemplateSrv {
     this.updateIndex();
   }
 
-  variableInitialized(variable) {
+  variableInitialized(variable: any) {
     this.index[variable.name] = variable;
   }
 
-  getAdhocFilters(datasourceName) {
-    let filters = [];
+  getAdhocFilters(datasourceName: string) {
+    let filters: any = [];
 
     if (this.variables) {
       for (let i = 0; i < this.variables.length; i++) {
@@ -92,7 +96,7 @@ export class TemplateSrv {
     return filters;
   }
 
-  luceneFormat(value) {
+  luceneFormat(value: any) {
     if (typeof value === 'string') {
       return luceneEscape(value);
     }
@@ -108,7 +112,7 @@ export class TemplateSrv {
   // encode string according to RFC 3986; in contrast to encodeURIComponent()
   // also the sub-delims "!", "'", "(", ")" and "*" are encoded;
   // unicode handling uses UTF-8 as in ECMA-262.
-  encodeURIComponentStrict(str) {
+  encodeURIComponentStrict(str: string) {
     return encodeURIComponent(str).replace(/[!'()*]/g, c => {
       return (
         '%' +
@@ -120,7 +124,7 @@ export class TemplateSrv {
     });
   }
 
-  formatValue(value, format, variable) {
+  formatValue(value: any, format: any, variable: any) {
     // for some scopedVars there is no variable
     variable = variable || {};
 
@@ -180,11 +184,11 @@ export class TemplateSrv {
     }
   }
 
-  setGrafanaVariable(name, value) {
+  setGrafanaVariable(name: string, value: any) {
     this.grafanaVariables[name] = value;
   }
 
-  getVariableName(expression) {
+  getVariableName(expression: string) {
     this.regex.lastIndex = 0;
     const match = this.regex.exec(expression);
     if (!match) {
@@ -194,12 +198,12 @@ export class TemplateSrv {
     return variableName;
   }
 
-  variableExists(expression) {
+  variableExists(expression: string) {
     const name = this.getVariableName(expression);
     return name && this.index[name] !== void 0;
   }
 
-  highlightVariablesAsHtml(str) {
+  highlightVariablesAsHtml(str: string) {
     if (!str || !_.isString(str)) {
       return str;
     }
@@ -214,7 +218,7 @@ export class TemplateSrv {
     });
   }
 
-  getAllValue(variable) {
+  getAllValue(variable: any) {
     if (variable.allValue) {
       return variable.allValue;
     }
@@ -225,21 +229,44 @@ export class TemplateSrv {
     return values;
   }
 
-  replace(target: string, scopedVars?: ScopedVars, format?: string | Function) {
+  getFieldAccessor(fieldPath: string) {
+    const accessor = this.fieldAccessorCache[fieldPath];
+    if (accessor) {
+      return accessor;
+    }
+
+    return (this.fieldAccessorCache[fieldPath] = _.property(fieldPath));
+  }
+
+  getVariableValue(variableName: string, fieldPath: string | undefined, scopedVars: ScopedVars) {
+    const scopedVar = scopedVars[variableName];
+    if (!scopedVar) {
+      return null;
+    }
+
+    if (fieldPath) {
+      return this.getFieldAccessor(fieldPath)(scopedVar.value);
+    }
+
+    return scopedVar.value;
+  }
+
+  replace(target: string, scopedVars?: ScopedVars, format?: string | Function): any {
     if (!target) {
       return target;
     }
 
-    let variable, systemValue, value, fmt;
     this.regex.lastIndex = 0;
 
-    return target.replace(this.regex, (match, var1, var2, fmt2, var3, fmt3) => {
-      variable = this.index[var1 || var2 || var3];
-      fmt = fmt2 || fmt3 || format;
+    return target.replace(this.regex, (match, var1, var2, fmt2, var3, fieldPath, fmt3) => {
+      const variableName = var1 || var2 || var3;
+      const variable = this.index[variableName];
+      const fmt = fmt2 || fmt3 || format;
+
       if (scopedVars) {
-        value = scopedVars[var1 || var2 || var3];
-        if (value) {
-          return this.formatValue(value.value, fmt, variable);
+        const value = this.getVariableValue(variableName, fieldPath, scopedVars);
+        if (value !== null && value !== undefined) {
+          return this.formatValue(value, fmt, variable);
         }
       }
 
@@ -247,12 +274,12 @@ export class TemplateSrv {
         return match;
       }
 
-      systemValue = this.grafanaVariables[variable.current.value];
+      const systemValue = this.grafanaVariables[variable.current.value];
       if (systemValue) {
         return this.formatValue(systemValue, fmt, variable);
       }
 
-      value = variable.current.value;
+      let value = variable.current.value;
       if (this.isAllValue(value)) {
         value = this.getAllValue(variable);
         // skip formatting of custom all values
@@ -266,11 +293,11 @@ export class TemplateSrv {
     });
   }
 
-  isAllValue(value) {
+  isAllValue(value: any) {
     return value === '$__all' || (Array.isArray(value) && value[0] === '$__all');
   }
 
-  replaceWithText(target, scopedVars) {
+  replaceWithText(target: string, scopedVars: ScopedVars) {
     if (!target) {
       return target;
     }
@@ -278,7 +305,7 @@ export class TemplateSrv {
     let variable;
     this.regex.lastIndex = 0;
 
-    return target.replace(this.regex, (match, var1, var2, fmt2, var3) => {
+    return target.replace(this.regex, (match: any, var1: any, var2: any, fmt2: any, var3: any) => {
       if (scopedVars) {
         const option = scopedVars[var1 || var2 || var3];
         if (option) {
@@ -297,7 +324,7 @@ export class TemplateSrv {
     });
   }
 
-  fillVariableValuesForUrl(params, scopedVars?) {
+  fillVariableValuesForUrl(params: any, scopedVars?: ScopedVars) {
     _.each(this.variables, variable => {
       if (scopedVars && scopedVars[variable.name] !== void 0) {
         if (scopedVars[variable.name].skipUrlSync) {
@@ -313,7 +340,7 @@ export class TemplateSrv {
     });
   }
 
-  distributeVariable(value, variable) {
+  distributeVariable(value: any, variable: any) {
     value = _.map(value, (val: any, index: number) => {
       if (index !== 0) {
         return variable + '=' + val;
