@@ -15,17 +15,20 @@ export interface DashboardDTO {
 export interface DashboardMeta {
   canSave?: boolean;
   canEdit?: boolean;
+  canDelete?: boolean;
   canShare?: boolean;
   canStar?: boolean;
   canAdmin?: boolean;
   url?: string;
   folderId?: number;
   fullscreen?: boolean;
+  fromExplore?: boolean;
   isEditing?: boolean;
   canMakeEditable?: boolean;
   submenuEnabled?: boolean;
   provisioned?: boolean;
-  focusPanelId?: boolean;
+  provisionedExternalId?: string;
+  focusPanelId?: number;
   isStarred?: boolean;
   showSettings?: boolean;
   expires?: string;
