@@ -1,7 +1,6 @@
 export * from './teams';
 export * from './alerting';
 export * from './location';
-export * from './navModel';
 export * from './folders';
 export * from './dashboard';
 export * from './acl';
@@ -14,3 +13,4 @@ export * from './appNotifications';
 export * from './search';
 export * from './explore';
 export * from './store';
+export * from './ldap';
